@@ -31,6 +31,8 @@ try:
         where,
         coarsen,
         insert,
+        delete,
+        append,
         shape,
         union1d,
         ravel,
@@ -56,6 +58,7 @@ try:
         flatnonzero,
         nonzero,
         unravel_index,
+        ravel_multi_index,
         around,
         isin,
         isnull,
@@ -82,6 +85,12 @@ try:
         fliplr,
         einsum,
         average,
+        triu,
+        tril,
+        tril_indices,
+        tril_indices_from,
+        triu_indices,
+        triu_indices_from,
     )
     from .reshape import reshape
     from .ufunc import (
@@ -230,8 +239,7 @@ try:
         indices,
         diag,
         eye,
-        triu,
-        tril,
+        tri,
         fromfunction,
         tile,
         repeat,
