@@ -3,7 +3,6 @@ import numpy as np
 
 import dask.array as da
 from dask.array.utils import assert_eq, IS_NEP18_ACTIVE
-from dask.array.numpy_compat import _numpy_120
 
 from .test_dispatch import EncapsulateNDArray, WrappedArray
 
@@ -16,6 +15,7 @@ missing_arrfunc_reason = "NEP-18 support is not available in NumPy"
 @pytest.mark.parametrize(
     "func",
     [
+        lambda x: np.append(x, x),
         lambda x: np.concatenate([x, x, x]),
         lambda x: np.cov(x, x),
         lambda x: np.dot(x, x),
@@ -33,6 +33,8 @@ missing_arrfunc_reason = "NEP-18 support is not available in NumPy"
         lambda x: np.min(x),
         lambda x: np.amin(x),
         lambda x: np.round(x),
+        lambda x: np.insert(x, 0, 3, axis=0),
+        lambda x: np.delete(x, 0, axis=0),
     ],
 )
 def test_array_function_dask(func):
@@ -92,7 +94,6 @@ def test_array_function_sparse(func):
 
 
 @pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
-@pytest.mark.xfail(_numpy_120, reason="sparse-383")
 def test_array_function_sparse_tensordot():
     sparse = pytest.importorskip("sparse")
     x = np.random.random((2, 3, 4))
@@ -109,11 +110,12 @@ def test_array_function_sparse_tensordot():
 
 
 @pytest.mark.skipif(missing_arrfunc_cond, reason=missing_arrfunc_reason)
-def test_array_function_cupy_svd():
+@pytest.mark.parametrize("chunks", [(100, 100), (500, 100)])
+def test_array_function_cupy_svd(chunks):
     cupy = pytest.importorskip("cupy")
     x = cupy.random.random((500, 100))
 
-    y = da.from_array(x, chunks=(100, 100), asarray=False)
+    y = da.from_array(x, chunks=chunks, asarray=False)
 
     u_base, s_base, v_base = da.linalg.svd(y)
     u, s, v = np.linalg.svd(y)
