@@ -149,3 +149,56 @@ def test_reshape_all_not_chunked_merge(
     result = a.reshape(outshape)
     assert result.chunks == outchunks
     assert_eq(result, base.reshape(outshape))
+
+
+@pytest.mark.parametrize(
+    "inshape, inchunks, outshape, outchunks",
+    [
+        # (2, 3, 4) -> (6, 4)
+        ((2, 3, 4), ((2,), (1, 2), (2, 2)), (6, 4), ((1, 2, 1, 2), (2, 2))),
+        # (1, 2, 3, 4) -> (12, 4)
+        ((1, 2, 3, 4), ((1,), (2,), (1, 2), (2, 2)), (6, 4), ((1, 2, 1, 2), (2, 2))),
+        # (2, 2, 3, 4) -> (12, 4) (3 cases)
+        (
+            (2, 2, 3, 4),
+            ((1, 1), (2,), (1, 2), (2, 2)),
+            (12, 4),
+            ((1, 2, 1, 2, 1, 2, 1, 2), (2, 2)),
+        ),
+        (
+            (2, 2, 3, 4),
+            ((2,), (1, 1), (1, 2), (2, 2)),
+            (12, 4),
+            ((1, 2, 1, 2, 1, 2, 1, 2), (2, 2)),
+        ),
+        (
+            (2, 2, 3, 4),
+            ((2,), (2,), (1, 2), (2, 2)),
+            (12, 4),
+            ((1, 2, 1, 2, 1, 2, 1, 2), (2, 2)),
+        ),
+        # (2, 2, 3, 4) -> (4, 3, 4)
+        # TODO: I'm confused about the behavior in this case.
+        # (
+        #     (2, 2, 3, 4),
+        #     ((2,), (2,), (1, 2), (2, 2)),
+        #     (4, 3, 4),
+        #     ((1, 1, 1, 1), (1, 2), (2, 2)),
+        # ),
+        # (2, 2, 3, 4) -> (4, 3, 4)
+        ((2, 2, 3, 4), ((2,), (2,), (1, 2), (4,)), (4, 3, 4), ((2, 2), (1, 2), (4,))),
+    ],
+)
+def test_reshape_merge_chunks(inshape, inchunks, outshape, outchunks):
+    # https://github.com/dask/dask/issues/5544#issuecomment-712280433
+    # When the early axes are completely chunked then we are just moving blocks
+    # and can avoid any rechunking. The outchunks will always be ...
+    base = np.arange(np.prod(inshape)).reshape(inshape)
+    a = da.from_array(base, chunks=inchunks)
+
+    # and via reshape
+    result = a.reshape(outshape, merge_chunks=False)
+    assert result.chunks == outchunks
+    assert_eq(result, base.reshape(outshape))
+
+    assert result.chunks != a.reshape(outshape).chunks
