@@ -1,10 +1,10 @@
 from collections import OrderedDict
-from collections.abc import Mapping, Iterator
 from contextlib import contextmanager
 from functools import partial
 from hashlib import md5
 from numbers import Number
 from operator import getitem
+from typing import Iterator, Mapping, Set
 import inspect
 import pickle
 import os
@@ -18,7 +18,7 @@ from .compatibility import is_dataclass, dataclass_fields
 from .context import thread_state
 from .core import flatten, quote, get as simple_get, literal
 from .hashing import hash_buffer_hex
-from .utils import Dispatch, ensure_dict, apply
+from .utils import Dispatch, ensure_dict, apply, key_split
 from . import config, local, threaded
 
 
@@ -32,6 +32,10 @@ __all__ = (
     "visualize",
     "tokenize",
     "normalize_token",
+    "get_collection_names",
+    "get_name_from_key",
+    "replace_name_in_key",
+    "clone_key",
 )
 
 
@@ -153,7 +157,7 @@ def is_dask_collection(x):
         return False
 
 
-class DaskMethodsMixin(object):
+class DaskMethodsMixin:
     """A mixin adding standard dask collection methods"""
 
     __slots__ = ()
@@ -314,38 +318,32 @@ def optimization_function(x):
     return getattr(x, "__dask_optimize__", dont_optimize)
 
 
-def collections_to_dsk(collections, optimize_graph=True, **kwargs):
+def collections_to_dsk(collections, optimize_graph=True, optimizations=(), **kwargs):
     """
     Convert many collections into a single dask graph, after optimization
     """
     from .highlevelgraph import HighLevelGraph
 
-    optimizations = kwargs.pop("optimizations", None) or config.get("optimizations", [])
+    optimizations = tuple(optimizations) + tuple(config.get("optimizations", ()))
 
     if optimize_graph:
         groups = groupby(optimization_function, collections)
 
-        _opt_list = []
+        graphs = []
         for opt, val in groups.items():
             dsk, keys = _extract_graph_and_keys(val)
-            groups[opt] = (dsk, keys)
-            _opt = opt(dsk, keys, **kwargs)
-            _opt_list.append(_opt)
+            dsk = opt(dsk, keys, **kwargs)
 
-        for opt in optimizations:
-            _opt_list = []
-            group = {}
-            for k, (dsk, keys) in groups.items():
-                _opt = opt(dsk, keys, **kwargs)
-                group[k] = (_opt, keys)
-                _opt_list.append(_opt)
-            groups = group
+            for opt in optimizations:
+                dsk = opt(dsk, keys, **kwargs)
+
+            graphs.append(dsk)
 
         # Merge all graphs
-        if any(isinstance(graph, HighLevelGraph) for graph in _opt_list):
-            dsk = HighLevelGraph.merge(*_opt_list)
+        if any(isinstance(graph, HighLevelGraph) for graph in graphs):
+            dsk = HighLevelGraph.merge(*graphs)
         else:
-            dsk = merge(*map(ensure_dict, _opt_list))
+            dsk = merge(*map(ensure_dict, graphs))
     else:
         dsk, _ = _extract_graph_and_keys(collections)
 
@@ -1192,3 +1190,98 @@ def wait(x, timeout=None, return_when="ALL_COMPLETED"):
         return wait(x, timeout=timeout, return_when=return_when)
     except (ImportError, ValueError):
         return x
+
+
+def get_collection_names(collection) -> Set[str]:
+    """Infer the collection names from the dask keys, under the assumption that all keys
+    are either tuples with matching first element, and that element is a string, or
+    there is exactly one key and it is a string.
+
+    Examples
+    --------
+    >>> a.__dask_keys__()  # doctest: +SKIP
+    ["foo", "bar"]  # doctest: +SKIP
+    >>> get_collection_names(a)  # doctest: +SKIP
+    {"foo", "bar"}  # doctest: +SKIP
+    >>> b.__dask_keys__()  # doctest: +SKIP
+    [[("foo-123", 0, 0), ("foo-123", 0, 1)], [("foo-123", 1, 0), ("foo-123", 1, 1)]]  # doctest: +SKIP
+    >>> get_collection_names(b)  # doctest: +SKIP
+    {"foo-123"}  # doctest: +SKIP
+    """
+    if not is_dask_collection(collection):
+        raise TypeError(f"Expected Dask collection; got {type(collection)}")
+    return {get_name_from_key(k) for k in flatten(collection.__dask_keys__())}
+
+
+def get_name_from_key(key) -> str:
+    """Given a dask collection's key, extract the collection name.
+
+    Parameters
+    ----------
+    key: string or tuple
+        Dask collection's key, which must be either a single string or a tuple whose
+        first element is a string (commonly referred to as a collection's 'name'),
+
+    Examples
+    --------
+    >>> get_name_from_key("foo")
+    'foo'
+    >>> get_name_from_key(("foo-123", 1, 2))
+    'foo-123'
+    """
+    if isinstance(key, tuple) and key and isinstance(key[0], str):
+        return key[0]
+    if isinstance(key, str):
+        return key
+    raise TypeError(f"Expected str or tuple[str, Hashable, ...]; got {key}")
+
+
+def replace_name_in_key(key, rename: Mapping[str, str]):
+    """Given a dask collection's key, replace the collection name with a new one.
+
+    Parameters
+    ----------
+    key: string or tuple
+        Dask collection's key, which must be either a single string or a tuple whose
+        first element is a string (commonly referred to as a collection's 'name'),
+    rename:
+        Mapping of zero or more names from : to. Extraneous names will be ignored.
+        Names not found in this mapping won't be replaced.
+
+    Examples
+    --------
+    >>> replace_name_in_key("foo", {})
+    'foo'
+    >>> replace_name_in_key("foo", {"foo": "bar"})
+    'bar'
+    >>> replace_name_in_key(("foo-123", 1, 2), {"foo-123": "bar-456"})
+    ('bar-456', 1, 2)
+    """
+    if isinstance(key, tuple) and key and isinstance(key[0], str):
+        return (rename.get(key[0], key[0]),) + key[1:]
+    if isinstance(key, str):
+        return rename.get(key, key)
+    raise TypeError(f"Expected str or tuple[str, Hashable, ...]; got {key}")
+
+
+def clone_key(key, seed):
+    """Clone a key from a Dask collection, producing a new key with the same prefix and
+    indices and a token which a deterministic function of the previous token and seed.
+
+    Examples
+    --------
+    >>> clone_key("inc-cbb1eca3bafafbb3e8b2419c4eebb387", 123)  # doctest: +SKIP
+    'inc-1d291de52f5045f8a969743daea271fd'  # doctest: +SKIP
+    >>> clone_key(("sum-cbb1eca3bafafbb3e8b2419c4eebb387", 4, 3), 123)  # doctest: +SKIP
+    ('sum-f0962cc58ef4415689a86cc1d4cc1723', 4, 3)  # doctest: +SKIP
+    """
+    if isinstance(key, tuple) and key and isinstance(key[0], str):
+        return (clone_key(key[0], seed),) + key[1:]
+    if isinstance(key, str):
+        prefix = key_split(key)
+        token = key[len(prefix) + 1 :]
+        if token:
+            return prefix + "-" + tokenize(token, seed)
+        else:
+            return tokenize(key, seed)
+    raise TypeError(f"Expected str or tuple[str, Hashable, ...]; got {key}")
