@@ -1,10 +1,13 @@
 import itertools
+import os
 import warnings
-
-import numpy as np
+from itertools import product
+from typing import Any, Hashable, Iterable, Mapping, Optional, Sequence, Set, Tuple
 
 import tlz as toolz
 
+from .base import clone_key, get_name_from_key
+from .compatibility import prod
 from .core import reverse_dict, flatten, keys_in_tasks
 from .delayed import unpack_collections
 from .highlevelgraph import HighLevelGraph, Layer
@@ -18,21 +21,6 @@ from .utils import (
 )
 
 
-class PackedFunctionCall:
-    """Function-decorator class to expand list arguments."""
-
-    def __init__(self, func):
-        self.func = func
-
-    def __call__(self, args):
-        if self.func is None:
-            return None
-        if isinstance(args, (tuple, list)):
-            return self.func(*args)
-        else:
-            return self.func(args)
-
-
 def subs(task, substitution):
     """Create a new task with the values substituted
 
@@ -70,12 +58,12 @@ def blockwise(
     concatenate=None,
     new_axes=None,
     dependencies=(),
-    **kwargs
+    **kwargs,
 ):
     """Create a Blockwise symbolic mutable mapping
 
-    This is like the ``make_blockwise_graph`` function, but rather than construct a dict, it
-    returns a symbolic Blockwise object.
+    This is like the ``make_blockwise_graph`` function, but rather than construct a
+    dict, it returns a symbolic Blockwise object.
 
     See Also
     --------
@@ -172,18 +160,19 @@ class Blockwise(Layer):
     dsk: dict
         A small graph to apply per-output-block.  May include keys from the
         input indices.
-    indices: Tuple[str, Tuple[str, str]]
+    indices: Tuple[str, Tuple[str, ...]]
         An ordered mapping from input key name, like ``'x'``
         to input indices, like ``('i', 'j')``
         Or includes literals, which have ``None`` for an index value
-    numblocks: Dict[key, Sequence[int]]
+    numblocks: Mapping[key, Sequence[int]]
         Number of blocks along each dimension for each input
-    concatenate: boolean
+    concatenate: bool
         Whether or not to pass contracted dimensions as a list of inputs or a
         single input to the block function
-    new_axes: Dict
-        New index dimensions that may have been created, and their extent
-    output_blocks: Set[Tuple]
+    new_axes: Mapping
+        New index dimensions that may have been created and their size,
+        e.g. ``{'j': 2, 'k': 3}``
+    output_blocks: Set[Tuple[int, ...]]
         Specify a specific set of required output blocks. Since the graph
         will only contain the necessary tasks to generate these outputs,
         this kwarg can be used to "cull" the abstract layer (without needing
@@ -198,17 +187,26 @@ class Blockwise(Layer):
     dask.array.blockwise
     """
 
+    output: str
+    output_indices: Tuple[str, ...]
+    dsk: Mapping[str, tuple]
+    indices: Tuple[Tuple[str, Optional[Tuple[str, ...]]], ...]
+    numblocks: Mapping[str, Sequence[int]]
+    concatenate: Optional[bool]
+    new_axes: Mapping[str, int]
+    output_blocks: Optional[Set[Tuple[int, ...]]]
+
     def __init__(
         self,
-        output,
-        output_indices,
-        dsk,
-        indices,
-        numblocks,
-        concatenate=None,
-        new_axes=None,
-        output_blocks=None,
-        annotations=None,
+        output: str,
+        output_indices: Iterable[str],
+        dsk: Mapping[str, tuple],
+        indices: Iterable[Tuple[str, Optional[Iterable[str]]]],
+        numblocks: Mapping[str, Sequence[int]],
+        concatenate: bool = None,
+        new_axes: Mapping[str, int] = None,
+        output_blocks: Set[Tuple[int, ...]] = None,
+        annotations: Mapping[str, Any] = None,
     ):
         super().__init__(annotations=annotations)
         self.output = output
@@ -221,22 +219,17 @@ class Blockwise(Layer):
         self.numblocks = numblocks
         # optimize_blockwise won't merge where `concatenate` doesn't match, so
         # enforce a canonical value if there are no axes for reduction.
-        if all(
-            all(i in output_indices for i in ind)
-            for name, ind in indices
+        output_indices_set = set(self.output_indices)
+        if concatenate is not None and all(
+            i in output_indices_set
+            for name, ind in self.indices
             if ind is not None
+            for i in ind
         ):
-            self.concatenate = None
-        else:
-            self.concatenate = concatenate
+            concatenate = None
+        self.concatenate = concatenate
         self.new_axes = new_axes or {}
 
-        # No IO subgraph allowed in `Blockwise`.
-        # Use `BlockwiseIO` to include IO. Note that the `io_name`
-        # attribute is only included in `Blockwise` to help
-        # simplify `_optimize_blockwise` logic.
-        self.io_name = None
-
     @property
     def dims(self):
         """Returns a dictionary mapping between each index specified in
@@ -293,7 +286,7 @@ class Blockwise(Layer):
         return iter(self._dict)
 
     def __len__(self):
-        return int(np.prod(list(self._out_numblocks().values())))
+        return int(prod(self._out_numblocks().values()))
 
     def _out_numblocks(self):
         d = {}
@@ -310,7 +303,9 @@ class Blockwise(Layer):
     def is_materialized(self):
         return hasattr(self, "_cached_dict")
 
-    def __dask_distributed_pack__(self, client):
+    def __dask_distributed_pack__(
+        self, all_hlg_keys, known_key_dependencies, client, client_keys
+    ):
         from distributed.worker import dumps_function
         from distributed.utils import CancelledError
         from distributed.utils_comm import unpack_remotedata
@@ -337,32 +332,39 @@ class Blockwise(Layer):
                 raise CancelledError(stringify(future.key))
 
         # All blockwise tasks will depend on the futures in `indices`
-        global_dependencies = tuple(stringify(f.key) for f in indices_unpacked_futures)
+        global_dependencies = {stringify(f.key) for f in indices_unpacked_futures}
 
-        ret = {
+        return {
             "output": self.output,
             "output_indices": self.output_indices,
             "func": func,
             "func_future_args": func_future_args,
             "global_dependencies": global_dependencies,
             "indices": indices,
+            "is_list": [isinstance(x, list) for x in indices],
             "numblocks": self.numblocks,
             "concatenate": self.concatenate,
             "new_axes": self.new_axes,
-            "annotations": self.pack_annotations(),
             "output_blocks": self.output_blocks,
             "dims": self.dims,
         }
 
-        return ret
-
     @classmethod
-    def __dask_distributed_unpack__(cls, state, dsk, dependencies, annotations):
-        raw, raw_deps = make_blockwise_graph(
+    def __dask_distributed_unpack__(cls, state, dsk, dependencies):
+        # Make sure we convert list items back from tuples in `indices`.
+        # The msgpack serialization will have converted lists into
+        # tuples, and tuples may be stringified during graph
+        # materialization (bad if the item was not a key).
+        indices = [
+            list(ind) if is_list else ind
+            for ind, is_list in zip(state["indices"], state["is_list"])
+        ]
+
+        layer_dsk, layer_deps = make_blockwise_graph(
             state["func"],
             state["output"],
             state["output_indices"],
-            *state["indices"],
+            *indices,
             new_axes=state["new_axes"],
             numblocks=state["numblocks"],
             concatenate=state["concatenate"],
@@ -372,16 +374,17 @@ class Blockwise(Layer):
             deserializing=True,
             func_future_args=state["func_future_args"],
         )
-        global_dependencies = list(state["global_dependencies"])
+        g_deps = state["global_dependencies"]
 
-        if state["annotations"]:
-            annotations.update(cls.expand_annotations(state["annotations"], raw.keys()))
-
-        raw = {stringify(k): stringify_collection_keys(v) for k, v in raw.items()}
-        dsk.update(raw)
-
-        for k, v in raw_deps.items():
-            dependencies[stringify(k)] = [stringify(d) for d in v] + global_dependencies
+        # Stringify layer graph and dependencies
+        layer_dsk = {
+            stringify(k): stringify_collection_keys(v) for k, v in layer_dsk.items()
+        }
+        deps = {
+            stringify(k): {stringify(d) for d in v} | g_deps
+            for k, v in layer_deps.items()
+        }
+        return {"dsk": layer_dsk, "deps": deps}
 
     def _cull_dependencies(self, all_hlg_keys, output_blocks):
         """Determine the necessary dependencies to produce `output_blocks`.
@@ -417,7 +420,7 @@ class Blockwise(Layer):
             deps = set()
             coords = out_coords + dummies
             for cmap, axes, (arg, ind) in zip(coord_maps, concat_axes, self.indices):
-                if ind is not None and arg != self.io_name:
+                if ind is not None:
                     arg_coords = tuple(coords[c] for c in cmap)
                     if axes:
                         tups = lol_product((arg,), arg_coords)
@@ -444,8 +447,9 @@ class Blockwise(Layer):
             annotations=self.annotations,
         )
 
-    def cull(self, keys, all_hlg_keys):
-
+    def cull(
+        self, keys: set, all_hlg_keys: Iterable
+    ) -> Tuple[Layer, Mapping[Hashable, set]]:
         # Culling is simple for Blockwise layers.  We can just
         # collect a set of required output blocks (tuples), and
         # only construct graph for these blocks in `make_blockwise_graph`
@@ -455,208 +459,70 @@ class Blockwise(Layer):
             if key[0] == self.output:
                 output_blocks.add(key[1:])
         culled_deps = self._cull_dependencies(all_hlg_keys, output_blocks)
-        out_size = tuple([self.dims[i] for i in self.output_indices])
-        if np.prod(out_size) != len(culled_deps):
+        out_size_iter = (self.dims[i] for i in self.output_indices)
+        if prod(out_size_iter) != len(culled_deps):
             culled_layer = self._cull(output_blocks)
             return culled_layer, culled_deps
         else:
             return self, culled_deps
 
-
-class BlockwiseIO(Blockwise):
-    """Blockwise layer with IO
-
-    This a specialized Blockwise layer containing the IO "subgraph"
-    required to construct a brand new collection.
-
-    Parameters
-    ----------
-    io_name: str
-        The name of the new collection to be created within this layer.
-        Note that this must match the name used in both `indices` and in
-        the keys of the `io_subgraph` input.
-    io_subgraph: Dict or Mapping
-        The graph needed to construct a new collection within this layer.
-        The `subgraph` must comprise of exactly N keys (where N is the
-        number of chunks/partitions in the new collection), and each value
-        must correspond to a "callable" task. The first element (a callable
-        function) must be the same for all N tasks.  This "uniformity" is
-        required for the abstract `SubgraphCallable` representation used
-        within Blockwise.
-    output: str
-        The name of the output collection.  Used in keynames
-    output_indices: tuple
-        The output indices, like ``('i', 'j', 'k')`` used to determine the
-        structure of the block computations
-    dsk: dict
-        A small graph to apply per-output-block.  May include keys from the
-        input indices.
-    indices: Tuple[str, Tuple[str, str]]
-        An ordered mapping from input key name, like ``'x'``
-        to input indices, like ``('i', 'j')``
-        Or includes literals, which have ``None`` for an index value
-    numblocks: Dict[key, Sequence[int]]
-        Number of blocks along each dimension for each input
-    concatenate: boolean
-        Whether or not to pass contracted dimensions as a list of inputs or a
-        single input to the block function
-    new_axes: Dict
-        New index dimensions that may have been created, and their extent
-    output_blocks: Set[Tuple]
-        Specify a specific set of required output blocks. Since the graph
-        will only contain the necessary tasks to generate these outputs,
-        this kwarg can be used to "cull" the abstract layer (without needing
-        to materialize the low-level graph).
-    annotations: dict (optional)
-        Layer annotations
-
-    See Also
-    --------
-    dask.blockwise.BlockwiseIO
-    """
-
-    def __init__(
+    def clone(
         self,
-        io_name,
-        io_subgraph,
-        output,
-        output_indices,
-        dsk,
-        indices,
-        numblocks,
-        concatenate=None,
-        new_axes=None,
-        output_blocks=None,
-        annotations=None,
-    ):
-        # Handle the case that this is a "pure" IO layer (dsk is None).
-        # Note that `dsk` should only be defined after fusion.
-        if not dsk:
-            # Extract actual IO function for SubgraphCallable construction.
-            # Wrap func in `PackedFunctionCall`, since it will receive
-            # all arguments as a sigle (packed) tuple at run time.
-            if io_subgraph:
-                # We assume a 1-to-1 mapping between keys (i.e. tasks) and
-                # chunks/partitions in `io_subgraph`, and assume the first
-                # (callable) element is the same for all tasks.
-                any_key = next(iter(io_subgraph))
-                io_func = io_subgraph.get(any_key)[0]
-            else:
-                io_func = None
-            ninds = 1 if isinstance(output_indices, str) else len(output_indices)
-            dsk = {
-                output: (
-                    PackedFunctionCall(io_func),
-                    *[blockwise_token(i) for i in range(ninds)],
-                )
-            }
+        keys: set,
+        seed: Hashable,
+        bind_to: Hashable = None,
+    ) -> Tuple[Layer, bool]:
+        names = {get_name_from_key(k) for k in keys}
+        # We assume that 'keys' will contain either all or none of the output keys of
+        # each of the layers, because clone/bind are always invoked at collection level.
+        # Asserting this is very expensive, so we only check it during unit tests.
+        if "PYTEST_CURRENT_TEST" in os.environ:
+            assert not self.get_output_keys() - keys
+            for name, nb in self.numblocks.items():
+                if name in names:
+                    for block in product(*(list(range(nbi)) for nbi in nb)):
+                        assert (name, *block) in keys
 
-        # Super-class initializer
-        super().__init__(
-            output,
-            output_indices,
-            dsk,
-            indices,
-            numblocks,
-            concatenate=None,
-            new_axes=None,
-            output_blocks=None,
-            annotations=annotations,
-        )
+        is_leaf = True
 
-        # BlockwiseIO requires `io_name` and `io_subgraph` inputs
-        self.io_name = io_name
-        self.io_subgraph = io_subgraph
+        indices = []
+        for k, idxv in self.indices:
+            if k in names:
+                is_leaf = False
+                k = clone_key(k, seed)
+            indices.append((k, idxv))
 
-    @property
-    def _dict(self):
-        if hasattr(self, "_cached_dict"):
-            return self._cached_dict["dsk"]
-        else:
-            keys = tuple(map(blockwise_token, range(len(self.indices))))
-            dsk, _ = fuse(self.dsk, [self.output])
-            func = SubgraphCallable(dsk, self.output, keys)
+        numblocks = {}
+        for k, nbv in self.numblocks.items():
+            if k in names:
+                is_leaf = False
+                k = clone_key(k, seed)
+            numblocks[k] = nbv
 
-            dsk = make_blockwise_graph(
-                func,
-                self.output,
-                self.output_indices,
-                *list(toolz.concat(self.indices)),
-                new_axes=self.new_axes,
-                numblocks=self.numblocks,
+        dsk = {clone_key(k, seed): v for k, v in self.dsk.items()}
+
+        if bind_to is not None and is_leaf:
+            from .graph_manipulation import chunks
+
+            # It's always a Delayed generated by dask.graph_manipulation.checkpoint;
+            # the layer name always matches the key
+            assert isinstance(bind_to, str)
+            dsk = {k: (chunks.bind, v, f"_{len(indices)}") for k, v in dsk.items()}
+            indices.append((bind_to, None))
+
+        return (
+            Blockwise(
+                output=clone_key(self.output, seed),
+                output_indices=self.output_indices,
+                dsk=dsk,
+                indices=indices,
+                numblocks=numblocks,
                 concatenate=self.concatenate,
+                new_axes=self.new_axes,
                 output_blocks=self.output_blocks,
-                dims=self.dims,
-            )
-
-            # Handle IO Subgraph
-            for k in dsk:
-                io_key = (self.io_name,) + tuple([k[i] for i in range(1, len(k))])
-                if io_key in dsk[k]:
-                    # Inject IO-function arguments into the blockwise graph
-                    # as a single (packed) tuple.
-                    io_item = self.io_subgraph.get(io_key)
-                    io_item = list(io_item[1:]) if len(io_item) > 1 else []
-                    new_task = [io_item if v == io_key else v for v in dsk[k]]
-                    dsk[k] = tuple(new_task)
-
-            self._cached_dict = {"dsk": dsk}
-        return self._cached_dict["dsk"]
-
-    def __dask_distributed_pack__(self, client):
-        ret = super().__dask_distributed_pack__(client)
-        ret["io_info"] = (self.io_name, self.io_subgraph)
-        return ret
-
-    @classmethod
-    def __dask_distributed_unpack__(cls, state, dsk, dependencies):
-        raw, raw_deps = make_blockwise_graph(
-            state["func"],
-            state["output"],
-            state["output_indices"],
-            *state["indices"],
-            new_axes=state["new_axes"],
-            numblocks=state["numblocks"],
-            concatenate=state["concatenate"],
-            output_blocks=state["output_blocks"],
-            dims=state["dims"],
-            return_key_deps=True,
-            deserializing=True,
-            func_future_args=state["func_future_args"],
-        )
-        io_name, io_subgraph = state["io_info"]
-        global_dependencies = list(state["global_dependencies"])
-
-        if io_subgraph:
-            # This is an IO layer.
-            for k in raw:
-                io_key = (io_name,) + tuple([k[i] for i in range(1, len(k))])
-                if io_key in raw[k]:
-                    # Inject IO-function arguments into the blockwise graph
-                    # as a single (packed) tuple.
-                    io_item = io_subgraph.get(io_key)
-                    io_item = list(io_item[1:]) if len(io_item) > 1 else []
-                    new_task = [io_item if v == io_key else v for v in raw[k]]
-                    raw[k] = tuple(new_task)
-
-        raw = {stringify(k): stringify_collection_keys(v) for k, v in raw.items()}
-        dsk.update(raw)
-
-        for k, v in raw_deps.items():
-            dependencies[stringify(k)] = [stringify(d) for d in v] + global_dependencies
-
-    def _cull(self, output_blocks):
-        return BlockwiseIO(
-            self.io_name,
-            self.io_subgraph,
-            self.output,
-            self.output_indices,
-            self.dsk,
-            self.indices,
-            self.numblocks,
-            concatenate=self.concatenate,
-            new_axes=self.new_axes,
-            output_blocks=output_blocks,
+                annotations=self.annotations,
+            ),
+            (bind_to is not None and is_leaf),
         )
 
 
@@ -1079,15 +945,14 @@ def optimize_blockwise(graph, keys=()):
 
 def _optimize_blockwise(full_graph, keys=()):
     keep = {k[0] if type(k) is tuple else k for k in keys}
-    layers = full_graph.dicts
+    layers = full_graph.layers
     dependents = reverse_dict(full_graph.dependencies)
-    roots = {k for k in full_graph.dicts if not dependents.get(k)}
+    roots = {k for k in full_graph.layers if not dependents.get(k)}
     stack = list(roots)
 
     out = {}
     dependencies = {}
     seen = set()
-    io_names = set()
 
     while stack:
         layer = stack.pop()
@@ -1099,7 +964,6 @@ def _optimize_blockwise(full_graph, keys=()):
         if isinstance(layers[layer], Blockwise):
             blockwise_layers = {layer}
             deps = set(blockwise_layers)
-            io_names.add(layers[layer].io_name)
             while deps:  # we gather as many sub-layers as we can
                 dep = deps.pop()
 
@@ -1155,7 +1019,7 @@ def _optimize_blockwise(full_graph, keys=()):
             for k, v in new_layer.indices:
                 if v is None:
                     new_deps |= keys_in_tasks(full_graph.dependencies, [k])
-                elif k not in io_names:
+                else:
                     new_deps.add(k)
             dependencies[layer] = new_deps
         else:
@@ -1210,7 +1074,6 @@ def rewrite_blockwise(inputs):
     concatenate = inputs[root].concatenate
     dsk = dict(inputs[root].dsk)
 
-    io_info = None
     changed = True
     while changed:
         changed = False
@@ -1222,10 +1085,6 @@ def rewrite_blockwise(inputs):
 
             changed = True
 
-            # Update IO-subgraph information
-            if not io_info and inputs[dep].io_name:
-                io_info = (inputs[dep].io_name, inputs[dep].io_subgraph)
-
             # Replace _n with dep name in existing tasks
             # (inc, _0) -> (inc, 'b')
             dsk = {k: subs(v, {blockwise_token(i): dep}) for k, v in dsk.items()}
@@ -1296,33 +1155,16 @@ def rewrite_blockwise(inputs):
     numblocks = toolz.merge([inp.numblocks for inp in inputs.values()])
     numblocks = {k: v for k, v in numblocks.items() if v is None or k in indices_check}
 
-    if io_info:
-        # Fused layer includes IO
-        out = BlockwiseIO(
-            *io_info,  # (io_name, io_subgraph)
-            root,
-            inputs[root].output_indices,
-            dsk,
-            new_indices,
-            numblocks=numblocks,
-            new_axes=new_axes,
-            concatenate=concatenate,
-            annotations=inputs[root].annotations,
-        )
-    else:
-        # Fused layer does NOT include IO
-        out = Blockwise(
-            root,
-            inputs[root].output_indices,
-            dsk,
-            new_indices,
-            numblocks=numblocks,
-            new_axes=new_axes,
-            concatenate=concatenate,
-            annotations=inputs[root].annotations,
-        )
-
-    return out
+    return Blockwise(
+        root,
+        inputs[root].output_indices,
+        dsk,
+        new_indices,
+        numblocks=numblocks,
+        new_axes=new_axes,
+        concatenate=concatenate,
+        annotations=inputs[root].annotations,
+    )
 
 
 def zero_broadcast_dimensions(lol, nblocks):
@@ -1449,8 +1291,8 @@ def fuse_roots(graph: HighLevelGraph, keys: list):
     Blockwise
     fuse
     """
-    layers = graph.layers.copy()
-    dependencies = graph.dependencies.copy()
+    layers = ensure_dict(graph.layers, copy=True)
+    dependencies = ensure_dict(graph.dependencies, copy=True)
     dependents = reverse_dict(dependencies)
 
     for name, layer in graph.layers.items():
@@ -1460,6 +1302,7 @@ def fuse_roots(graph: HighLevelGraph, keys: list):
             and len(deps) > 1
             and not any(dependencies[dep] for dep in deps)  # no need to fuse if 0 or 1
             and all(len(dependents[dep]) == 1 for dep in deps)
+            and all(layer.annotations == graph.layers[dep].annotations for dep in deps)
         ):
             new = toolz.merge(layer, *[layers[dep] for dep in deps])
             new, _ = fuse(new, keys, ave_width=len(deps))
