@@ -9,9 +9,8 @@ unicode = str  # noqa
 
 try:
     from dataclasses import is_dataclass, fields as dataclass_fields
-
 except ImportError:
-
+    # Python < 3.7
     def is_dataclass(x):
         return False
 
@@ -19,4 +18,15 @@ except ImportError:
         return []
 
 
+try:
+    from math import prod
+except ImportError:
+    # Python < 3.8
+    def prod(iterable, *, start=1):
+        acc = start
+        for el in iterable:
+            acc *= el
+        return acc
+
+
 PY_VERSION = LooseVersion(".".join(map(str, sys.version_info[:3])))
