@@ -9,7 +9,6 @@ from .utils import (
     is_series_like,
     is_index_like,
     is_dataframe_like,
-    PANDAS_GT_0250,
     hash_object_dispatch,
     group_split_dispatch,
 )
@@ -129,6 +128,13 @@ def wrap_var_reduction(array_var, index):
     return array_var
 
 
+def wrap_skew_reduction(array_skew, index):
+    if isinstance(array_skew, np.ndarray) or isinstance(array_skew, list):
+        return pd.Series(array_skew, index=index)
+
+    return array_skew
+
+
 def var_mixed_concat(numeric_var, timedelta_var, columns):
     vars = pd.concat([numeric_var, timedelta_var])
 
@@ -198,10 +204,9 @@ def describe_nonnumeric_aggregate(stats, name):
         data = [0, 0]
         index = ["count", "unique"]
         dtype = None
-        if PANDAS_GT_0250:
-            data.extend([None, None])
-            index.extend(["top", "freq"])
-            dtype = object
+        data.extend([None, None])
+        index.extend(["top", "freq"])
+        dtype = object
         result = pd.Series(data, index=index, dtype=dtype, name=name)
         return result
 
@@ -231,7 +236,7 @@ def describe_nonnumeric_aggregate(stats, name):
 
 
 def _cum_aggregate_apply(aggregate, x, y):
-    """ Apply aggregation function within a cumulative aggregation
+    """Apply aggregation function within a cumulative aggregation
 
     Parameters
     ----------
@@ -247,6 +252,24 @@ def _cum_aggregate_apply(aggregate, x, y):
         return aggregate(x, y)
 
 
+def cumsum_aggregate(x, y):
+    if x is None:
+        return y
+    elif y is None:
+        return x
+    else:
+        return x + y
+
+
+def cumprod_aggregate(x, y):
+    if x is None:
+        return y
+    elif y is None:
+        return x
+    else:
+        return x * y
+
+
 def cummin_aggregate(x, y):
     if is_series_like(x) or is_dataframe_like(x):
         return x.where((x < y) | x.isnull(), y, axis=x.ndim - 1)
@@ -338,7 +361,7 @@ def pivot_agg(df):
 
 def pivot_sum(df, index, columns, values):
     return pd.pivot_table(
-        df, index=index, columns=columns, values=values, aggfunc="sum"
+        df, index=index, columns=columns, values=values, aggfunc="sum", dropna=False
     )
 
 
@@ -346,7 +369,7 @@ def pivot_count(df, index, columns, values):
     # we cannot determine dtype until concatenationg all partitions.
     # make dtype deterministic, always coerce to np.float64
     return pd.pivot_table(
-        df, index=index, columns=columns, values=values, aggfunc="count"
+        df, index=index, columns=columns, values=values, aggfunc="count", dropna=False
     ).astype(np.float64)
 
 
