@@ -120,7 +120,9 @@ class SimpleShuffleLayer(Layer):
         ]
         return (SimpleShuffleLayer, tuple(getattr(self, attr) for attr in attrs))
 
-    def __dask_distributed_pack__(self, client):
+    def __dask_distributed_pack__(
+        self, all_hlg_keys, known_key_dependencies, client, client_keys
+    ):
         from distributed.protocol.serialize import to_serialize
 
         return {
@@ -132,11 +134,10 @@ class SimpleShuffleLayer(Layer):
             "name_input": self.name_input,
             "meta_input": to_serialize(self.meta_input),
             "parts_out": list(self.parts_out),
-            "annotations": self.pack_annotations(),
         }
 
     @classmethod
-    def __dask_distributed_unpack__(cls, state, dsk, dependencies, annotations):
+    def __dask_distributed_unpack__(cls, state, dsk, dependencies):
         from distributed.worker import dumps_task
 
         # msgpack will convert lists into tuples, here
@@ -147,19 +148,18 @@ class SimpleShuffleLayer(Layer):
             state["inputs"] = list(state["inputs"])
 
         # Materialize the layer
-        raw = dict(cls(**state))
+        layer_dsk = dict(cls(**state))
 
         # Convert all keys to strings and dump tasks
-        raw = {stringify(k): stringify_collection_keys(v) for k, v in raw.items()}
-        dsk.update(toolz.valmap(dumps_task, raw))
+        layer_dsk = {
+            stringify(k): stringify_collection_keys(v) for k, v in layer_dsk.items()
+        }
+        keys = layer_dsk.keys() | dsk.keys()
 
         # TODO: use shuffle-knowledge to calculate dependencies more efficiently
-        dependencies.update(
-            {k: keys_in_tasks(dsk, [v], as_list=True) for k, v in raw.items()}
-        )
+        deps = {k: keys_in_tasks(keys, [v]) for k, v in layer_dsk.items()}
 
-        if state["annotations"]:
-            annotations.update(cls.expand_annotations(state["annotations"], raw.keys()))
+        return {"dsk": toolz.valmap(dumps_task, layer_dsk), "deps": deps}
 
     def _keys_to_parts(self, keys):
         """Simple utility to convert keys to partition indices."""
@@ -341,8 +341,8 @@ class ShuffleLayer(SimpleShuffleLayer):
 
         return (ShuffleLayer, tuple(getattr(self, attr) for attr in attrs))
 
-    def __dask_distributed_pack__(self, client):
-        ret = super().__dask_distributed_pack__(client)
+    def __dask_distributed_pack__(self, *args, **kwargs):
+        ret = super().__dask_distributed_pack__(*args, **kwargs)
         ret["inputs"] = self.inputs
         ret["stage"] = self.stage
         ret["nsplits"] = self.nsplits
@@ -359,8 +359,12 @@ class ShuffleLayer(SimpleShuffleLayer):
         for part in parts_out:
             out = self.inputs[part]
             for k in range(self.nsplits):
-                _part = inp_part_map[insert(out, self.stage, k)]
-                deps[(self.name, part)].add((self.name_input, _part))
+                _inp = insert(out, self.stage, k)
+                _part = inp_part_map[_inp]
+                if self.stage == 0 and _part >= self.npartitions_input:
+                    deps[(self.name, part)].add(("group-" + self.name, _inp, "empty"))
+                else:
+                    deps[(self.name, part)].add((self.name_input, _part))
         return deps
 
     def _cull(self, parts_out):
@@ -519,6 +523,7 @@ def set_index(
             mins == sorted(mins)
             and maxes == sorted(maxes)
             and all(mx < mn for mx, mn in zip(maxes[:-1], mins[1:]))
+            and npartitions == df.npartitions
         ):
             divisions = mins + [maxes[-1]]
             result = set_sorted_index(df, index, drop=drop, divisions=divisions)
@@ -684,7 +689,7 @@ def shuffle(
         else:
             index = list(index)
         nset = set(index)
-        if nset.intersection(set(df.columns)) == nset:
+        if nset & set(df.columns) == nset:
             return rearrange_by_column(
                 df,
                 index,
@@ -764,7 +769,7 @@ def rearrange_by_column(
         raise NotImplementedError("Unknown shuffle method %s" % shuffle)
 
 
-class maybe_buffered_partd(object):
+class maybe_buffered_partd:
     """
     If serialized, will return non-buffered partd. Otherwise returns a buffered partd
     """
@@ -850,22 +855,14 @@ def rearrange_by_column_disk(df, column, npartitions=None, compute=False):
     dsk3 = {barrier_token: (barrier, list(dsk2))}
 
     # Collect groups
-    name1 = "shuffle-collect-1" + token
+    name = "shuffle-collect-" + token
     dsk4 = {
-        (name1, i): (collect, p, i, df._meta, barrier_token) for i in range(npartitions)
+        (name, i): (collect, p, i, df._meta, barrier_token) for i in range(npartitions)
     }
-    cleanup_token = "cleanup-" + always_new_token
-    barrier_token2 = "barrier2-" + always_new_token
-    # A task that depends on `cleanup-`, but has a small output
-    dsk5 = {(barrier_token2, i): (barrier, part) for i, part in enumerate(dsk4)}
-    # This indirectly depends on `cleanup-` and so runs after we're done using the disk
-    dsk6 = {cleanup_token: (cleanup_partd_files, p, list(dsk5))}
 
-    name = "shuffle-collect-2" + token
-    dsk7 = {(name, i): (_noop, (name1, i), cleanup_token) for i in range(npartitions)}
     divisions = (None,) * (npartitions + 1)
 
-    layer = toolz.merge(dsk1, dsk2, dsk3, dsk4, dsk5, dsk6, dsk7)
+    layer = toolz.merge(dsk1, dsk2, dsk3, dsk4)
     graph = HighLevelGraph.from_collections(name, layer, dependencies=dependencies)
     return DataFrame(graph, name, df._meta, divisions)
 
@@ -1226,11 +1223,27 @@ def fix_overlap(ddf, overlap):
     n = len(ddf.divisions) - 1
     dsk = {(name, i): (ddf._name, i) for i in range(n)}
 
+    frames = []
     for i in overlap:
-        frames = [(get_overlap, (ddf._name, i - 1), ddf.divisions[i]), (ddf._name, i)]
-        dsk[(name, i)] = (methods.concat, frames)
+
+        # `frames` is a list of data from previous partitions that we may want to
+        # move to partition i.  Here, we add "overlap" from the previous partition
+        # (i-1) to this list.
+        frames.append((get_overlap, (ddf._name, i - 1), ddf.divisions[i]))
+
+        # Make sure that any data added from partition i-1 to `frames` is removed
+        # from partition i-1.
         dsk[(name, i - 1)] = (drop_overlap, dsk[(name, i - 1)], ddf.divisions[i])
 
+        # We do not want to move "overlap" from the previous partition (i-1) into
+        # this partition (i) if the data from this partition will need to be moved
+        # to the next partitition (i+1) anyway.  If we concatenate data too early,
+        # we may loose rows (https://github.com/dask/dask/issues/6972).
+        if i == ddf.npartitions - 2 or ddf.divisions[i] != ddf.divisions[i + 1]:
+            frames.append((ddf._name, i))
+            dsk[(name, i)] = (methods.concat, frames)
+            frames = []
+
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=[ddf])
     return new_dd_object(graph, name, ddf._meta, ddf.divisions)
 
