@@ -2,9 +2,9 @@ from decimal import Decimal
 import pytest
 
 import dask.dataframe as dd
-from dask.dataframe.utils import assert_eq, PANDAS_VERSION
+from dask.dataframe.utils import assert_eq
 
-pd = pytest.importorskip("pandas", minversion="0.23.4")
+pd = pytest.importorskip("pandas")
 
 from pandas.tests.extension.decimal.array import DecimalArray, DecimalDtype
 from dask.dataframe.extensions import make_array_nonempty, make_scalar
@@ -12,11 +12,7 @@ from dask.dataframe.extensions import make_array_nonempty, make_scalar
 
 @make_array_nonempty.register(DecimalDtype)
 def _(dtype):
-    kwargs = {}
-    if PANDAS_VERSION >= "0.24.0rc1":
-        kwargs["dtype"] = dtype
-
-    return DecimalArray._from_sequence([Decimal("0"), Decimal("NaN")], **kwargs)
+    return DecimalArray._from_sequence([Decimal("0"), Decimal("NaN")], dtype=dtype)
 
 
 @make_scalar.register(Decimal)
@@ -36,7 +32,6 @@ def test_register_extension_type():
 
 
 def test_reduction():
-    pytest.importorskip("pandas", minversion="0.24.0")
     ser = pd.Series(DecimalArray._from_sequence([Decimal("0"), Decimal("1")]))
     dser = dd.from_pandas(ser, 2)
     assert_eq(ser.mean(skipna=False), dser.mean(skipna=False))
