@@ -17,6 +17,9 @@ from dask.dataframe.utils import (
     assert_max_deps,
 )
 
+# TODO(pandas) Categorical.set_levels(inplace=True) is deprecated
+pytestmark = pytest.mark.filterwarnings("ignore:inplace is deprecated:FutureWarning")
+
 AGG_FUNCS = [
     "sum",
     "mean",
@@ -2228,25 +2231,19 @@ def test_groupby_aggregate_categoricals(grouping, agg):
     assert_eq(agg(grouping(pdf)["value"]), agg(grouping(ddf)["value"]))
 
 
-@pytest.mark.xfail(reason="dropna kwarg not supported in pandas groupby.")
+@pytest.mark.xfail(
+    not dask.dataframe.utils.PANDAS_GT_110,
+    reason="dropna kwarg not supported in pandas < 1.1.0.",
+)
 @pytest.mark.parametrize("dropna", [False, True])
 def test_groupby_dropna_pandas(dropna):
-
-    # The `dropna` arg is not currently supported by pandas
-    # (See #https://github.com/pandas-dev/pandas/pull/21669)
-    # Dask supports the argument for the cudf backend,
-    # but passing it to the pandas backend will fail.
-
-    # TODO: Expand test when `dropna` is supported in pandas.
-    #       (See: `test_groupby_dropna_cudf`)
-
     df = pd.DataFrame(
         {"a": [1, 2, 3, 4, None, None, 7, 8], "e": [4, 5, 6, 3, 2, 1, 0, 0]}
     )
     ddf = dd.from_pandas(df, npartitions=3)
 
-    dask_result = ddf.groupby("a", dropna=dropna)
-    pd_result = df.groupby("a", dropna=dropna)
+    dask_result = ddf.groupby("a", dropna=dropna).e.sum()
+    pd_result = df.groupby("a", dropna=dropna).e.sum()
     assert_eq(dask_result, pd_result)
 
 
@@ -2285,6 +2282,37 @@ def test_groupby_dropna_cudf(dropna, by):
     assert_eq(dask_result, cudf_result)
 
 
+@pytest.mark.xfail(
+    not dask.dataframe.utils.PANDAS_GT_110,
+    reason="Should work starting from pandas 1.1.0",
+)
+def test_groupby_dropna_with_agg():
+    # https://github.com/dask/dask/issues/6986
+    df = pd.DataFrame(
+        {"id1": ["a", None, "b"], "id2": [1, 2, None], "v1": [4.5, 5.5, None]}
+    )
+    expected = df.groupby(["id1", "id2"], dropna=False).agg("sum")
+
+    ddf = dd.from_pandas(df, 1, sort=False)
+    actual = ddf.groupby(["id1", "id2"], dropna=False).agg("sum")
+    assert_eq(expected, actual)
+
+
+def test_groupby_observed_with_agg():
+    df = pd.DataFrame(
+        {
+            "cat_1": pd.Categorical(list("AB"), categories=list("ABCDE")),
+            "cat_2": pd.Categorical([1, 2], categories=[1, 2, 3]),
+            "value_1": np.random.uniform(size=2),
+        }
+    )
+    expected = df.groupby(["cat_1", "cat_2"], observed=True).agg("sum")
+
+    ddf = dd.from_pandas(df, 2)
+    actual = ddf.groupby(["cat_1", "cat_2"], observed=True).agg("sum")
+    assert_eq(expected, actual)
+
+
 def test_rounding_negative_var():
     x = [-0.00179999999 for _ in range(10)]
     ids = [1 for _ in range(5)] + [2 for _ in range(5)]
@@ -2450,3 +2478,26 @@ def test_groupby_aggregate_categorical_observed(
         agg(pdf.groupby(groupby, observed=observed)),
         agg(ddf.groupby(groupby, observed=observed)),
     )
+
+
+def test_empty_partitions_with_value_counts():
+    # https://github.com/dask/dask/issues/7065
+    df = pd.DataFrame(
+        data=[
+            ["a1", "b1"],
+            ["a1", None],
+            ["a1", "b1"],
+            [None, None],
+            [None, None],
+            [None, None],
+            ["a3", "b3"],
+            ["a3", "b3"],
+            ["a5", "b5"],
+        ],
+        columns=["A", "B"],
+    )
+
+    expected = df.groupby("A")["B"].value_counts()
+    ddf = dd.from_pandas(df, npartitions=3)
+    actual = ddf.groupby("A")["B"].value_counts()
+    assert_eq(expected, actual)
