@@ -66,6 +66,16 @@ def ddf_right_single(df_right):
     return dd.from_pandas(df_right, npartitions=1, sort=False)
 
 
+@pytest.fixture
+def ddf_right_double(df_right):
+    return dd.from_pandas(df_right, npartitions=2, sort=False)
+
+
+@pytest.fixture
+def ddf_left_double(df_left):
+    return dd.from_pandas(df_left, npartitions=2, sort=False)
+
+
 @pytest.fixture(params=["inner", "left", "right", "outer"])
 def how(request):
     return request.param
@@ -162,3 +172,40 @@ def test_merge_unknown_to_unknown(
     assert_eq(result, expected)
     assert_eq(result.divisions, tuple(None for _ in range(11)))
     assert len(result.__dask_graph__()) >= 390
+
+
+@pytest.mark.parametrize("how", ["inner", "left"])
+def test_merge_known_to_double_bcast_right(
+    df_left, df_right, ddf_left, ddf_right_double, on, how
+):
+    # Compute expected
+    expected = df_left.merge(df_right, on=on, how=how)
+
+    # Perform merge
+    result = ddf_left.merge(
+        ddf_right_double, on=on, how=how, shuffle="tasks", broadcast=True
+    )
+
+    # Assertions
+    assert_eq(result, expected)
+    assert_eq(result.divisions, ddf_left.divisions)
+    assert len(result.__dask_graph__()) < 90
+
+
+@pytest.mark.parametrize("how", ["inner", "right"])
+@pytest.mark.parametrize("broadcast", [True, 0.75])
+def test_merge_known_to_double_bcast_left(
+    df_left, df_right, ddf_left_double, ddf_right, on, how, broadcast
+):
+    # Compute expected
+    expected = df_left.merge(df_right, on=on, how=how)
+
+    # Perform merge
+    result = ddf_left_double.merge(
+        ddf_right, on=on, how=how, shuffle="tasks", broadcast=broadcast
+    )
+
+    # Assertions
+    assert_eq(result, expected)
+    assert_eq(result.divisions, ddf_right.divisions)
+    assert len(result.__dask_graph__()) < 90
