@@ -16,8 +16,8 @@ from dask.dataframe.utils import (
     is_dataframe_like,
     is_series_like,
     is_index_like,
-    PANDAS_GT_0240,
     PANDAS_GT_100,
+    PANDAS_GT_120,
 )
 
 import pytest
@@ -242,11 +242,7 @@ def test_meta_nonempty_index():
 
     levels = [pd.Int64Index([1], name="a"), pd.Float64Index([1.0], name="b")]
     codes = [[0], [0]]
-    if PANDAS_GT_0240:
-        kwargs = {"codes": codes}
-    else:
-        kwargs = {"labels": codes}
-    idx = pd.MultiIndex(levels=levels, names=["a", "b"], **kwargs)
+    idx = pd.MultiIndex(levels=levels, names=["a", "b"], codes=codes)
     res = meta_nonempty(idx)
     assert type(res) is pd.MultiIndex
     for idx1, idx2 in zip(idx.levels, res.levels):
@@ -261,12 +257,8 @@ def test_meta_nonempty_index():
     ]
 
     codes = [[0], [0], [0]]
-    if PANDAS_GT_0240:
-        kwargs = {"codes": codes}
-    else:
-        kwargs = {"labels": codes}
 
-    idx = pd.MultiIndex(levels=levels, names=["a", "b", "timedelta"], **kwargs)
+    idx = pd.MultiIndex(levels=levels, names=["a", "b", "timedelta"], codes=codes)
     res = meta_nonempty(idx)
     assert type(res) is pd.MultiIndex
     for idx1, idx2 in zip(idx.levels, res.levels):
@@ -433,6 +425,32 @@ def test_is_dataframe_like(monkeypatch, frame_value_counts):
     assert is_index_like(ddf.index)
     assert not is_index_like(pd.Index)
 
+    # The following checks support of class wrappers, which
+    # requires the comparions of `x.__class__` instead of `type(x)`
+    class DataFrameWrapper:
+        __class__ = pd.DataFrame
+
+    wrap = DataFrameWrapper()
+    wrap.dtypes = None
+    wrap.columns = None
+    assert is_dataframe_like(wrap)
+
+    class SeriesWrapper:
+        __class__ = pd.Series
+
+    wrap = SeriesWrapper()
+    wrap.dtype = None
+    wrap.name = None
+    assert is_series_like(wrap)
+
+    class IndexWrapper:
+        __class__ = pd.Index
+
+    wrap = IndexWrapper()
+    wrap.dtype = None
+    wrap.name = None
+    assert is_index_like(wrap)
+
 
 def test_apply_and_enforce_message():
     def func():
@@ -453,3 +471,10 @@ def test_nonempty_series_sparse():
         dd.utils._nonempty_series(ser)
 
     assert len(w) == 0
+
+
+@pytest.mark.skipif(not PANDAS_GT_120, reason="Float64 was introduced in pandas>=1.2")
+def test_nonempty_series_nullable_float():
+    ser = pd.Series([], dtype="Float64")
+    non_empty = dd.utils._nonempty_series(ser)
+    assert non_empty.dtype == "Float64"
