@@ -214,7 +214,7 @@ class _Tracker(Process):
     """Background process for tracking resource usage"""
 
     def __init__(self, dt=1):
-        Process.__init__(self)
+        super().__init__()
         self.daemon = True
         self.dt = dt
         self.parent_pid = current_process().pid
@@ -347,7 +347,7 @@ class CacheProfiler(Callback):
     def _posttask(self, key, value, dsk, state, id):
         t = default_timer()
         self._cache[key] = (self._metric(value), t)
-        for k in state["released"].intersection(self._cache):
+        for k in state["released"] & self._cache.keys():
             metric, start = self._cache.pop(k)
             self.results.append(CacheData(k, dsk[k], metric, start, t))
 
