@@ -3,9 +3,6 @@ import sys
 import multiprocessing
 from operator import add
 import pickle
-import random
-
-import numpy as np
 
 import pytest
 import dask
@@ -28,6 +25,7 @@ not_cloudpickle = pytest.mark.skipif(has_cloudpickle, reason="cloudpickle is ins
 
 
 def unrelated_function_global(a):
+    np = pytest.importorskip("numpy")
     return np.array([a])
 
 
@@ -46,6 +44,7 @@ def test_pickle_globals():
 @requires_cloudpickle
 def test_pickle_locals():
     """Unrelated locals should not be included in serialized bytes"""
+    np = pytest.importorskip("numpy")
 
     def unrelated_function_local(a):
         return np.array([a])
@@ -78,6 +77,7 @@ def test_pickle_kwargs():
 @pytest.mark.skipif(pickle.HIGHEST_PROTOCOL < 5, reason="requires pickle protocol 5")
 def test_out_of_band_pickling():
     """Test that out-of-band pickling works"""
+    np = pytest.importorskip("numpy")
     if has_cloudpickle:
         if cloudpickle.__version__ < LooseVersion("1.3.0"):
             pytest.skip("when using cloudpickle, it must be version 1.3.0+")
@@ -151,7 +151,7 @@ def test_lambda_results_without_cloudpickle():
     assert "cloudpickle" in str(e.value)
 
 
-class NotUnpickleable(object):
+class NotUnpickleable:
     def __getstate__(self):
         return ()
 
@@ -201,8 +201,37 @@ def test_optimize_graph_false():
 
 
 @requires_cloudpickle
-@pytest.mark.parametrize("random", [np.random, random])
+def test_works_with_highlevel_graph():
+    """Previously `dask.multiprocessing.get` would accidentally forward
+    `HighLevelGraph` graphs through the dask optimization/scheduling routines,
+    resulting in odd errors. One way to trigger this was to have a
+    non-indexable object in a task. This is just a smoketest to ensure that
+    things work properly even if `HighLevelGraph` objects get passed to
+    `dask.multiprocessing.get`. See https://github.com/dask/dask/issues/7190.
+    """
+
+    class NoIndex:
+        def __init__(self, x):
+            self.x = x
+
+        def __getitem__(self, key):
+            raise Exception("Oh no!")
+
+    x = delayed(lambda x: x)(NoIndex(1))
+    (res,) = get(x.dask, x.__dask_keys__())
+    assert isinstance(res, NoIndex)
+    assert res.x == 1
+
+
+@requires_cloudpickle
+@pytest.mark.parametrize("random", ["numpy", "random"])
 def test_random_seeds(random):
+    if random == "numpy":
+        np = pytest.importorskip("numpy")
+        random = np.random
+    else:
+        import random
+
     @delayed(pure=False)
     def f():
         return tuple(random.randint(0, 10000) for i in range(5))
