@@ -791,3 +791,89 @@ def test_order_with_equal_dependents(abcde):
     for x in abc:
         for i in range(len(abc)):
             assert o[(x, 5, i, 1)] - o[(x, 5, i, 0)] == 1
+
+
+def test_terminal_node_backtrack():
+    r"""
+    https://github.com/dask/dask/issues/6745
+
+    We have
+
+    1. A terminal node that depends on the entire graph ('s')
+    2. Some shared dependencies near the roots ('a1', 'a4')
+    3. But the left and right halves are disconnected, other
+       than the terminal node.
+
+                       s
+               /   /       \   \
+              /   /         \   \
+            s00  s10       s01  s11
+             |    |         |    |
+            b00  b10       b01  b11
+            / \  / \       / \ / \
+           a0  a1  a2    a3  a4  a5
+
+    Previously we started at 'a', and worked up to 's00'. We'd like to finish
+    's00' completely, so we progress to 's' and work through its dependencies.
+
+    Ideally, we would choose 's10', since we've already computed one of its
+    (eventual) dependencies: 'a1'. However, all of 's00' through 's11' had
+    equal metrics so we fell back to the name tie-breaker and started on
+    's01' (via 'a3', a4', 'b01', ...).
+    """
+    dsk = {
+        # left half
+        ("a", 0): (0,),
+        ("a", 1): (1,),
+        ("a", 2): (2,),
+        ("b", 0): (f, ("a", 0), ("a", 1)),
+        ("b", 1): (f, ("a", 1), ("a", 2)),
+        ("store", 0, 0): ("b", 0),
+        ("store", 1, 0): ("b", 1),
+        # right half
+        ("a", 3): (3,),
+        ("a", 4): (4,),
+        ("a", 5): (5,),
+        ("b", 2): (f, ("a", 3), ("a", 4)),
+        ("b", 3): (f, ("a", 4), ("a", 5)),
+        ("store", 0, 1): ("b", 2),
+        ("store", 1, 1): ("b", 3),
+        "store": (
+            f,
+            ("store", 0, 0),
+            ("store", 1, 0),
+            ("store", 0, 1),
+            ("store", 1, 1),
+        ),
+    }
+    o = order(dsk)
+    assert o[("a", 2)] < o[("a", 3)]
+
+
+def test_array_store_final_order(tmpdir):
+    # https://github.com/dask/dask/issues/6745
+    # This essentially tests the same thing as test_terminal_node_backtrack,
+    # but with the graph actually generated by da.store.
+    da = pytest.importorskip("dask.array")
+    zarr = pytest.importorskip("zarr")
+    import numpy as np
+
+    arrays = [da.from_array(np.ones((110, 4)), chunks=(100, 2)) for i in range(4)]
+    x = da.concatenate(arrays, axis=0).rechunk((100, 2))
+
+    store = zarr.DirectoryStore(tmpdir)
+    root = zarr.group(store, overwrite=True)
+    dest = root.empty_like(name="dest", data=x, chunks=x.chunksize, overwrite=True)
+    d = x.store(dest, lock=False, compute=False)
+    o = order(d.dask)
+
+    # Find the lowest store. Dask starts here.
+    stores = [k for k in o if isinstance(k, tuple) and k[0].startswith("store-")]
+    first_store = min(stores, key=lambda k: o[k])
+    first_store
+    connected_stores = [k for k in stores if k[-1] == first_store[-1]]
+    disconnected_stores = [k for k in stores if k[-1] != first_store[-1]]
+
+    connected_max = max([v for k, v in o.items() if k in connected_stores])
+    disconnected_min = min([v for k, v in o.items() if k in disconnected_stores])
+    assert connected_max < disconnected_min
