@@ -3,9 +3,9 @@ import functools
 import operator
 import pickle
 
-import numpy as np
 import pytest
 
+from dask import get
 from dask.utils import (
     getargspec,
     takes_multiple_arguments,
@@ -29,9 +29,12 @@ from dask.utils import (
     parse_bytes,
     is_arraylike,
     iter_chunks,
+    stringify,
+    stringify_collection_keys,
 )
 from dask.utils_test import inc
 from dask.highlevelgraph import HighLevelGraph
+from dask.optimization import SubgraphCallable
 
 
 def test_getargspec():
@@ -50,7 +53,7 @@ def test_getargspec():
     wrapper.__wrapped__ = func
     assert getargspec(wrapper).args == ["x", "y"]
 
-    class MyType(object):
+    class MyType:
         def __init__(self, x, y):
             pass
 
@@ -64,11 +67,11 @@ def test_takes_multiple_arguments():
     def multi(a, b, c):
         return a, b, c
 
-    class Singular(object):
+    class Singular:
         def __init__(self, a):
             pass
 
-    class Multi(object):
+    class Multi:
         def __init__(self, a, b):
             pass
 
@@ -100,7 +103,7 @@ def test_dispatch():
 
     foo.register(object, f)
 
-    class Bar(object):
+    class Bar:
         pass
 
     b = Bar()
@@ -153,6 +156,7 @@ def test_dispatch_lazy():
 
 
 def test_random_state_data():
+    np = pytest.importorskip("numpy")
     seed = 37
     state = np.random.RandomState(seed)
     n = 10000
@@ -340,7 +344,7 @@ def test_funcname():
     assert funcname(M.sum) == "sum"
     assert funcname(lambda: 1) == "lambda"
 
-    class Foo(object):
+    class Foo:
         pass
 
     assert funcname(Foo) == "Foo"
@@ -413,17 +417,18 @@ def test_ndeepmap():
 def test_ensure_dict():
     d = {"x": 1}
     assert ensure_dict(d) is d
-    hlg = HighLevelGraph.from_collections("x", d)
-    assert type(ensure_dict(hlg)) is dict
-    assert ensure_dict(hlg) == d
 
     class mydict(dict):
         pass
 
-    md = mydict()
-    md["x"] = 1
-    assert type(ensure_dict(md)) is dict
-    assert ensure_dict(md) == d
+    d2 = ensure_dict(d, copy=True)
+    d3 = ensure_dict(HighLevelGraph.from_collections("x", d))
+    d4 = ensure_dict(mydict(d))
+
+    for di in (d2, d3, d4):
+        assert type(di) is dict
+        assert di is not d
+        assert di == d
 
 
 def test_itemgetter():
@@ -558,6 +563,8 @@ def test_parse_timedelta():
 
 
 def test_is_arraylike():
+    np = pytest.importorskip("numpy")
+
     assert is_arraylike(0) is False
     assert is_arraylike(()) is False
     assert is_arraylike(0) is False
@@ -582,3 +589,52 @@ def test_iter_chunks():
     ]
     assert list(iter_chunks(sizes, 28)) == [[14, 8, 5], [9, 7, 9, 1], [19, 8], [19]]
     assert list(iter_chunks(sizes, 67)) == [[14, 8, 5, 9, 7, 9, 1], [19, 8, 19]]
+
+
+def test_stringify():
+    obj = "Hello"
+    assert stringify(obj) is obj
+    obj = b"Hello"
+    assert stringify(obj) is obj
+    dsk = {"x": 1}
+
+    assert stringify(dsk) == str(dsk)
+    assert stringify(dsk, exclusive=()) == dsk
+
+    dsk = {("x", 1): (inc, 1)}
+    assert stringify(dsk) == str({("x", 1): (inc, 1)})
+    assert stringify(dsk, exclusive=()) == {("x", 1): (inc, 1)}
+
+    dsk = {("x", 1): (inc, 1), ("x", 2): (inc, ("x", 1))}
+    assert stringify(dsk, exclusive=dsk) == {
+        ("x", 1): (inc, 1),
+        ("x", 2): (inc, str(("x", 1))),
+    }
+
+    dsks = [
+        {"x": 1},
+        {("x", 1): (inc, 1), ("x", 2): (inc, ("x", 1))},
+        {("x", 1): (sum, [1, 2, 3]), ("x", 2): (sum, [("x", 1), ("x", 1)])},
+    ]
+    for dsk in dsks:
+        sdsk = {stringify(k): stringify(v, exclusive=dsk) for k, v in dsk.items()}
+        keys = list(dsk)
+        skeys = [str(k) for k in keys]
+        assert all(isinstance(k, str) for k in sdsk)
+        assert get(dsk, keys) == get(sdsk, skeys)
+
+    dsk = {("y", 1): (SubgraphCallable({"x": ("y", 1)}, "x", (("y", 1),)), (("z", 1),))}
+    dsk = stringify(dsk, exclusive=set(dsk) | {("z", 1)})
+    assert dsk[("y", 1)][0].dsk["x"] == "('y', 1)"
+    assert dsk[("y", 1)][1][0] == "('z', 1)"
+
+
+def test_stringify_collection_keys():
+    obj = "Hello"
+    assert stringify_collection_keys(obj) is obj
+
+    obj = [("a", 0), (b"a", 0), (1, 1)]
+    res = stringify_collection_keys(obj)
+    assert res[0] == str(obj[0])
+    assert res[1] == str(obj[1])
+    assert res[2] == obj[2]
